"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const wt = require("@waves/waves-transactions");
const waves_transactions_1 = require("@waves/waves-transactions/");
const ride_js_1 = require("@waves/ride-js");
const { keyPair, address } = waves_transactions_1.libs.crypto;
class WavesConsoleAPI {
    constructor() {
        this.alias = WavesConsoleAPI.injectEnv(wt.alias);
        this.burn = WavesConsoleAPI.injectEnv(wt.burn);
        this.cancelLease = WavesConsoleAPI.injectEnv(wt.cancelLease);
        this.cancelOrder = WavesConsoleAPI.injectEnv(wt.cancelOrder);
        this.data = WavesConsoleAPI.injectEnv(wt.data);
        this.issue = WavesConsoleAPI.injectEnv(wt.issue);
        this.reissue = WavesConsoleAPI.injectEnv(wt.reissue);
        this.lease = WavesConsoleAPI.injectEnv(wt.lease);
        this.massTransfer = WavesConsoleAPI.injectEnv(wt.massTransfer);
        this.order = WavesConsoleAPI.injectEnv(wt.order);
        this.transfer = WavesConsoleAPI.injectEnv(wt.transfer);
        this.setScript = WavesConsoleAPI.injectEnv(wt.setScript);
        this.setAssetScript = WavesConsoleAPI.injectEnv(wt.setAssetScript);
        this.contractInvocation = WavesConsoleAPI.injectEnv(wt.contractInvocation);
        this.signTx = WavesConsoleAPI.injectEnv(wt.signTx);
        this.addressBalance = (address, apiBase) => wt.addressBalance(address, apiBase || WavesConsoleAPI.env.API_BASE);
        this.addressDataByKey = (address, key, apiBase) => wt.addressDataByKey(address, key, apiBase || WavesConsoleAPI.env.API_BASE);
        this.broadcast = (tx, apiBase) => wt.broadcast(tx, apiBase || WavesConsoleAPI.env.API_BASE);
        this.file = (tabName) => {
            if (typeof WavesConsoleAPI.env.file !== 'function') {
                throw new Error('File content API is not available. Please provide it to the console');
            }
            return WavesConsoleAPI.env.file(tabName);
        };
        this.contract = () => this.file();
        this.keyPair = (seed) => keyPair(seed || WavesConsoleAPI.env.SEED);
        this.publicKey = (seed) => this.keyPair(seed).public;
        this.privateKey = (seed) => this.keyPair(seed).private;
        this.address = (seed, chainId) => address(seed || WavesConsoleAPI.env.SEED, chainId || WavesConsoleAPI.env.CHAIN_ID);
        this.compile = (code) => {
            const resultOrError = ride_js_1.compile(code);
            if ('error' in resultOrError)
                throw new Error(resultOrError.error);
            return resultOrError.result.base64;
        };
        this.deploy = (params, seed) => __awaiter(this, void 0, void 0, function* () {
            let txParams = Object.assign({ additionalFee: 400000, script: this.compile(this.contract()) }, params);
            const setScriptTx = this['setScript'](txParams, seed);
            return this['broadcast'](setScriptTx);
        });
        this.help = (func) => {
            let pos = -1, al0 = '', type = typeof func, params = [], aliases = [];
            // Try to find function name
            for (al0 in this) {
                if ((type == 'undefined' || func == this[al0])) {
                    aliases.push(al0);
                }
            }
            // Sort functions list and move help help to the top
            if (aliases.length > 1) {
                aliases.sort((a, b) => {
                    if (a > b) {
                        return 1;
                    }
                    else if (a < b) {
                        return -1;
                    }
                    return 0;
                });
                // Get position of help in list
                pos = aliases.indexOf('help');
                // Move help to the top of list
                aliases.unshift(aliases.splice(pos, 1)[0]);
            }
            // Compile help text from pieces
            return WavesConsoleAPIHelp.compileText(aliases);
        };
    }
    static setEnv(env) {
        WavesConsoleAPI.env = env;
    }
    bufferToBase64(buf) {
        const binstr = Array.prototype.map.call(buf, (ch) => String.fromCharCode(ch)).join('');
        return btoa(binstr);
    }
}
WavesConsoleAPI.injectEnv = (f) => (po, seed) => f(Object.assign({ chainId: WavesConsoleAPI.env.CHAIN_ID }, po), seed === null ? null : seed || WavesConsoleAPI.env.SEED);
exports.WavesConsoleAPI = WavesConsoleAPI;
/**
 * Help parsers and text compilers
 *
 * @static
 * @class WavesConsoleAPIHelp
 */
class WavesConsoleAPIHelp {
    /**
     * Generates API method argument(s) whole description
     *
     * @static
     * @method compileText
     *
     * @param {Array} aliases
     *
     * @returns {string}
     */
    static compileText(aliases) {
        let last = aliases.length - 1, module = WavesConsoleAPIHelp, full = aliases.length === 1, text = '';
        // Compile text for each command
        aliases.forEach((alias, index) => {
            text = this.compileTextSlice(alias, full, text);
            // Add ; or .
            if (!full) {
                if (index == last) {
                    text = `${text}.`;
                }
                else {
                    text = `${text};`;
                }
            }
        });
        // Add header for commands list
        if (full === false) {
            text = `${module.common.list.header}\n${text}`;
        }
        return text;
    }
    /**
     * Generates API method arguments description
     *
     * @static
     * @method compileTextArguments
     *
     * @param {Array} args
     * @param {string} text
     *
     * @returns {string}
     */
    static compileTextArguments(args, text) {
        let last = args.length - 1, type = '', summary = '', module = WavesConsoleAPIHelp;
        // Add arguments list header
        text = `${text}\n\n${module.common.args.header}`;
        // Add each argument info
        args.forEach((argument, index) => {
            text = `${text}\n${index + 1}. ${argument}`;
            if (module.types[argument]) {
                summary = module.types[argument].summary;
                summary = summary.substring(0, 1).toLowerCase() + summary.substring(1);
                type = module.types[argument].type;
                type = type ? type : '';
                // Add argument type
                if (type) {
                    text = `${text}: ${type}`;
                }
                // Add argument summary
                if (summary) {
                    text = `${text} — ${summary}`;
                }
                // Add ; or .
                if (index == last) {
                    text = `${text}.`;
                }
                else {
                    text = `${text};`;
                }
            }
        });
        return text;
    }
    /**
     * Generates API specified method description
     *
     * @static
     * @method compileTextSlice
     *
     * @param {string} alias
     * @param {boolean} full
     * @param {string} text
     *
     * @returns {string}
     */
    static compileTextSlice(alias, full, text) {
        let module = WavesConsoleAPIHelp, summary = '', params = module.texts[alias] && module.texts[alias].params ?
            module.texts[alias].params :
            [], description = '', args = params.slice();
        // Check optional and obligatory function params
        if (args) {
            args = args.map((arg) => {
                return module.types[arg].optional ? `[${arg}]` : `${arg}`;
            });
        }
        // Add common function info
        if (full) {
            text = `${alias}(${args.join(', ')})`;
        }
        else {
            text = `${text}\n${alias}(${args.join(', ')})`;
        }
        if (module.texts[alias]) {
            // Add summary text
            if (module.texts[alias].summary) {
                summary = module.texts[alias].summary;
                summary = summary.substring(0, 1).toLowerCase() +
                    summary.substring(1);
                text = `${text} — ${summary}`;
                if (full) {
                    text = `${text}.`;
                }
            }
            // Add arguments description
            if (full && args.length) {
                text = module.compileTextArguments(params, text);
            }
            // Add full description
            if (full && module.texts[alias].description) {
                description = module.texts[alias].description;
                text = `${text}\n\n${description}`;
            }
        }
        return text;
    }
}
/**
 * Common texts pieces
 *
 * @static
 * @member {object} common
 */
WavesConsoleAPIHelp.common = {
    list: {
        header: 'Available functions:'
    },
    args: {
        header: 'Arguments:'
    }
};
/**
 * Commands descriptions vocabulary
 *
 * @static
 * @member {object} texts
 */
WavesConsoleAPIHelp.texts = {
    addressBalance: {
        summary: '' +
            'Retrieve information about waves account balance',
        description: '' +
            'Returns Promise<number>.',
        params: ['address', 'nodeUrl']
    },
    addressDataByKey: {
        summary: '' +
            'Get data from account dictionary by key',
        description: '' +
            'Returns Promise<number | Uint8Array | string | null>.',
        params: ['address', 'key', 'nodeUrl']
    },
    clear: {
        summary: '' +
            'clear console;',
        description: '' +
            '',
        params: null
    },
    file: {
        summary: '' +
            'Gets editor contents for tab',
        description: '' +
            'Used inside web-ide or vscode plugin.',
        params: ['tabName']
    },
    data: {
        summary: '' +
            'Creates signed data transaction',
        description: '' +
            'You can use this function with multiple seeds. ' +
            'In this case it will sign transaction accordingly ' +
            'and will add one proof per seed. Also you can use ' +
            'already signed DataTransaction as a second argument.',
        params: ['params', 'seed']
    },
    issue: {
        summary: '' +
            'Creates signed issue transaction',
        description: '' +
            'You can use this function with multiple seeds. ' +
            'In this case it will sign transaction accordingly ' +
            'and will add one proof per seed. Also you can use ' +
            'already signed IssueTransaction as a second argument.',
        params: ['params', 'seed']
    },
    order: {
        summary: '' +
            'Creates and signs Order for exchange transactions',
        description: '' +
            'You can use this function with multiple seeds. ' +
            'In this case it will sign order accordingly and will ' +
            'add one proof per seed. Also you can use already signed ' +
            'Order as a second argument.',
        params: ['params', 'seed']
    },
    contract: {
        summary: '' +
            'Open editor tab content',
        description: '' +
            '',
        params: null
    },
    keyPair: {
        summary: '' +
            'Generates keyPair from seed',
        description: '' +
            '',
        params: ['seed']
    },
    publicKey: {
        summary: '' +
            'Generates publicKey from seed',
        description: '' +
            '',
        params: ['seed']
    },
    privateKey: {
        summary: '' +
            'Generates privateKey from seed',
        description: '' +
            '',
        params: ['seed']
    },
    address: {
        summary: '' +
            'Generates address from seed',
        description: '' +
            '',
        params: ['seed']
    },
    compile: {
        summary: '' +
            'Gets editor contents for tab',
        description: '' +
            'Accepts plain text of a contract as an argument. Returns compiled contract in base64.',
        params: ['code']
    },
    broadcast: {
        summary: '' +
            'Sends transaction to the Waves network using REST API',
        description: '' +
            'Returns Promise.',
        params: ['tx', 'apiBase']
    },
    deploy: {
        summary: '' +
            'Compile currently selected contract and deploy it to default account',
        description: '' +
            '',
        params: ['params', 'seed']
    },
    help: {
        summary: '' +
            'Help for the available API functions',
        description: '' +
            'You can use help() to get list of available functions ' +
            'or help(functionName) to get info for the specified function.',
        params: ['func']
    },
    transfer: {
        summary: '' +
            'Creates signed transfer transaction',
        description: '' +
            '',
        params: ['params', 'seed']
    },
    massTransfer: {
        summary: '' +
            'Creates signed massTransfer transaction',
        description: '' +
            'You can use this function with multiple seeds. ' +
            'In this case it will sign transaction accordingly ' +
            'and will add one proof per seed. Also you can use ' +
            'already signed MassTransferTransaction as a second argument.',
        params: ['params', 'seed']
    },
    reissue: {
        summary: '' +
            'Creates signed reissue transaction',
        description: '' +
            'You can use this function with multiple seeds. ' +
            'In this case it will sign transaction accordingly ' +
            'and will add one proof per seed. Also you can use already ' +
            'signed ReissueTransaction as a second argument.',
        params: ['params', 'seed']
    },
    burn: {
        summary: '' +
            'Creates signed burn transaction',
        description: '' +
            'You can use this function with multiple seeds. ' +
            'In this case it will sign transaction accordingly ' +
            'and will add one proof per seed. Also you can use ' +
            'already signed BurnTransaction as a second argument.',
        params: ['params', 'seed']
    },
    lease: {
        summary: '' +
            'Creates signed lease transaction',
        description: '' +
            'You can use this function with multiple seeds. ' +
            'In this case it will sign transaction accordingly ' +
            'and will add one proof per seed. Also you can use ' +
            'already signed LeaseTransaction as a second argument.',
        params: ['params', 'seed']
    },
    cancelLease: {
        summary: '' +
            'Creates signed cancelLease transaction',
        description: '' +
            'You can use this function with multiple seeds. ' +
            'In this case it will sign transaction accordingly ' +
            'and will add one proof per seed. Also you can use ' +
            'already signed CancelLeaseTransaction as a second argument.',
        params: ['params', 'seed']
    },
    cancelOrder: {
        summary: '' +
            'Creates signed cancelOrder request',
        description: '' +
            '',
        params: ['params', 'seed']
    },
    alias: {
        summary: '' +
            'Creates signed createAlias transaction',
        description: '' +
            'You can use this function with multiple seeds. ' +
            'In this case it will sign transaction accordingly ' +
            'and will add one proof per seed. Also you can use ' +
            'already signed AliasTransaction as a second argument.',
        params: ['params', 'seed']
    },
    setScript: {
        summary: '' +
            'Creates signed setScript transaction',
        description: '' +
            'You can use this function with multiple seeds. ' +
            'In this case it will sign transaction accordingly ' +
            'and will add one proof per seed. Also you can use ' +
            'already signed SetScriptTransaction as a second argument.',
        params: ['params', 'seed']
    },
    setAssetScript: {
        summary: '' +
            'Creates signed setAssetScript transaction',
        description: '' +
            'You can use this function with multiple seeds. ' +
            'In this case it will sign transaction accordingly ' +
            'and will add one proof per seed. Also you can use ' +
            'already signed SetAssetScriptTransaction as a second argument.',
        params: ['params', 'seed']
    },
    contractInvocation: {
        summary: '' +
            'Creates signed contractInvocation transaction',
        description: '' +
            'You can use this function with multiple seeds. ' +
            'In this case it will sign transaction accordingly ' +
            'and will add one proof per seed. Also you can use ' +
            'already signed CancelLeaseTransaction as a second argument.',
        params: ['params', 'seed']
    },
    signTx: {
        summary: '' +
            'Signs previously created transaction',
        description: '' +
            'You can use this function with multiple seeds. ' +
            'In this case it will sign transaction accordingly ' +
            'and will add one proof per seed',
        params: ['tx', 'seed']
    }
};
/**
 * Variables types vocabulary
 *
 * @static
 * @member {object} types
 */
WavesConsoleAPIHelp.types = {
    tx: {
        summary: 'Transaction object obtained from WavesTransactions library',
        type: 'object'
    },
    code: {
        summary: 'Text of the contract',
        type: 'string'
    },
    seed: {
        optional: true,
        summary: 'Seed string obtained from node (optional, env.SEED by default)',
        type: 'string'
    },
    func: {
        summary: 'Name of the function from API',
        type: 'function'
    },
    params: {
        summary: 'Object with transactions properties',
        type: 'object'
    },
    tabName: {
        summary: 'Tab name for web-ide or vscode plugin',
        type: 'string'
    },
    keyPairOrSeed: {
        summary: 'Seed string or keyPair object from keyPair() function',
        type: 'string'
    },
    apiBase: {
        optional: true,
        summary: 'Url of the node. E.x. "https://nodes.wavesplatform.com". (optional, env.API_BASE by default)',
        type: 'string'
    },
    nodeUrl: {
        optional: true,
        summary: 'Url of the node. E.x. "https://nodes.wavesplatform.com". (optional, env.API_BASE by default)',
        type: 'string'
    },
    address: {
        summary: 'Waves address as base58 string',
        type: 'string'
    },
    key: {
        summary: 'Account data dictionary key',
        type: 'string'
    }
};
exports.WavesConsoleAPIHelp = WavesConsoleAPIHelp;
