"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const WavesConsoleAPI_1 = require("../../WavesConsoleAPI");
const utils_1 = require("../utils");
// TODO import Autocomplete from './Autocomplete';
const keycodes_1 = require("../lib/keycodes");
class Input extends React.Component {
    constructor(props) {
        super(props);
        // history is set in the componentDidMount
        this.state = {
            value: props.value || '',
            multiline: false,
            rows: 1,
            historyCursor: props.history.length,
            hideSuggest: false
        };
        // Bind some methods to instance
        this.onChange = this.onChange.bind(this);
        this.onKeyPress = this.onKeyPress.bind(this);
    }
    onChange() {
        if (!this.input) {
            return;
        }
        const { value } = this.input;
        const length = value.split('\n').length;
        this.setState({
            multiline: length > 1,
            rows: length < 20 ? length : 20,
            value,
        });
    }
    onKeyPress(event) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.input) {
                return;
            }
            ;
            const code = keycodes_1.default[event.keyCode];
            const { multiline } = this.state;
            const { history } = this.props;
            const command = this.input.value;
            // Clear console
            if (this.checkClearAction(event, code)) {
                return;
            }
            // Insert value from suggested commands
            if (this.checkShowSuggestAction(event, code)) {
                return;
            }
            // Insert closing bracket if needed
            this.checkAutoclosingAction(event);
            // Move in history if not in multiline mode
            if (!multiline && this.checkNotMultilineActions(event, code, this.input)) {
                return;
            }
            // Remove suggestions block if needed
            if (this.checkHideSuggestAction(event, code)) {
                return;
            }
            // Add command to history and try to execute it
            if (code === 'enter') {
                if (event.shiftKey) {
                    return;
                }
                if (!command) {
                    event.preventDefault();
                    return;
                }
                this.props.addHistory(command);
                this.setState({ historyCursor: history.length + 1, value: '', multiline: false });
                event.preventDefault();
                yield this.props.onRun(command);
                // Don't use `this.input.scrollIntoView();` as it messes with iframes
                window.scrollTo(0, document.body.scrollHeight);
                return;
            }
        });
    }
    checkClearAction(event, code) {
        // Clear console
        if (event.ctrlKey && code === 'l') {
            this.props.onClear();
            return true;
        }
        return false;
    }
    checkShowSuggestAction(event, code) {
        if (code == 'tab') {
            event.preventDefault();
            this.setCommandIntoInput();
            return true;
        }
        return false;
    }
    checkHideSuggestAction(event, code) {
        if (code === 'escape') {
            event.preventDefault();
            this.setState({ hideSuggest: true });
            return true;
        }
        else {
            this.setState({ hideSuggest: false });
        }
        return false;
    }
    checkAutoclosingAction(event) {
        switch (event.key) {
            case '{':
            case '[':
            case '(':
                this.setClosingBracketIntoInput(event.key);
                break;
            case '}':
            case ']':
            case ')':
                this.setCaretAfterClosingBracket(event);
                break;
            case '"':
            case "'":
                this.setClosingQuoteOrSetCaretAfterClosingQuateIntoInput(event);
                break;
            case 'Backspace':
                this.unsetClosingBracketOrQuoteIntoInput();
                break;
        }
    }
    checkNotMultilineActions(event, code, input) {
        const { history } = this.props;
        let { historyCursor } = this.state;
        // Show back
        if (code === 'up arrow') {
            historyCursor--;
            let rows = history[historyCursor] ? history[historyCursor].split('\n') : [""];
            if (historyCursor < 0) {
                this.setState({ historyCursor: 0 });
                return true;
            }
            if (rows.length === 1 || (input.selectionStart <= rows[0].length && input.selectionEnd <= rows[0].length)) {
                this.setState({
                    historyCursor,
                    value: history[historyCursor]
                }, () => {
                    input.setSelectionRange(rows[0].length, rows[0].length);
                });
            }
            else
                return false;
            event.preventDefault();
            return true;
        }
        // Move forward
        if (code === 'down arrow') {
            let len = history[historyCursor] ? history[historyCursor].length : 0;
            historyCursor++;
            if (historyCursor >= history.length && input.selectionStart === len && input.selectionEnd === len) {
                this.setState({ historyCursor: history.length, value: '' });
                return true;
            }
            if (input.selectionStart === len && input.selectionEnd === len) {
                let len = history[historyCursor] ? history[historyCursor].length : 0;
                this.setState({
                    historyCursor,
                    value: history[historyCursor]
                }, () => { input.setSelectionRange(len, len); });
            }
            else
                return false;
            event.preventDefault();
            return true;
        }
        return false;
    }
    getCurrentCommandPiece() {
        let { input } = this;
        let pos = input ? input.selectionStart : 0;
        let commands = (input ? input.value.substring(0, pos) : '').
            split(/[\s+()]/);
        // Get last entry from string
        if (commands && commands.length) {
            return commands.pop();
        }
        return '';
    }
    setClosingBracketIntoInput(open = '(') {
        // No need to go further
        if (!this.input) {
            return;
        }
        let brackets = Input.commasAndQuotes;
        let { input } = this;
        let pos = input.selectionStart || 0;
        let close = brackets[open] ? brackets[open] : brackets['('];
        // Set new value
        input.value = input.value.substring(0, pos) +
            close +
            input.value.substring(pos);
        // Set new caret position
        this.setInputCaretPosition(pos);
        // Re-render to cleanup
        this.setInputValue(input.value);
    }
    setCaretAfterClosingBracket(event) {
        let { input } = this;
        // No need to go further
        if (!input) {
            return;
        }
        ;
        let pos = input.selectionStart || 0;
        let open = input.value.substr(pos - 1, 1);
        let close = Input.commasAndQuotes[open];
        // Check if the closing symbol is similar to needed
        close = input.value.substr(pos, 1) === close ? close : '';
        // No need to go further
        if (!close) {
            return;
        }
        ;
        event.preventDefault();
        // Set new caret position
        this.setInputCaretPosition(pos + 1);
        // // Re-render to cleanup
        this.setInputValue(input.value);
    }
    setClosingQuoteOrSetCaretAfterClosingQuateIntoInput(event) {
        let { input } = this;
        // No need to go further
        if (!input) {
            return;
        }
        ;
        let pos = input.selectionStart || 0;
        let open = input.value.substr(pos - 1, 1);
        let close = input.value.substr(pos, 1);
        let key = event.key;
        if (open === key && close === key) {
            event.preventDefault();
            this.setInputCaretPosition(pos + 1);
            // Re-render to cleanup
            this.setInputValue(input.value);
        }
        else {
            input.value = input.value.substring(0, pos) +
                key +
                input.value.substring(pos);
            // Set new caret position
            this.setInputCaretPosition(pos);
            // // Re-render to cleanup
            this.setInputValue(input.value);
        }
    }
    unsetClosingBracketOrQuoteIntoInput() {
        // No need to go further
        if (!this.input) {
            return;
        }
        let { input } = this;
        let pos = input.selectionStart || 0;
        // No need to go further
        if (pos === 0) {
            return;
        }
        let open = this.input.value.substr(pos - 1, 1);
        let close = Input.commasAndQuotes[open];
        // No need to go further
        if (!close) {
            return;
        }
        // Check if the closing symbol is similar to needed
        close = this.input.value.substr(pos, 1) === close ? close : '';
        // No need to go further
        if (!close) {
            return;
        }
        // Set new value
        input.value = input.value.substring(0, pos) +
            input.value.substring(pos + 1);
        // Set new caret position
        this.setInputCaretPosition(pos);
        // Re-render to cleanup
        this.setInputValue(input.value);
    }
    setCommandIntoInput() {
        // No need to go further
        if (!this.input) {
            return;
        }
        ;
        let { input } = this;
        let beg = input.selectionStart || 0;
        let end = input.selectionEnd || 0;
        let pos = beg;
        let insert = this.getCurrentCommandPiece();
        let missing = '';
        let commands = this.getFilteredCommandsList();
        if (insert === undefined) {
            return;
        }
        ;
        // If only one command
        if (commands.length === 1) {
            let isFunc = false;
            let command = '';
            let vocabulary = Input.commandsVocabulary;
            // Get command
            command = commands[0];
            // No need to go further
            if (command === undefined) {
                return;
            }
            // Check if it's method or member
            isFunc = vocabulary[command] && vocabulary[command].params !== undefined;
            // Get missing part of command name
            missing = command.substring(insert.length);
            // Set new value
            input.value = input.value.substring(0, beg) +
                missing + (isFunc ? '()' : '') +
                input.value.substring(end);
            // Set new caret position
            pos += missing.length;
            pos += isFunc ? 1 : 0;
            this.setInputCaretPosition(pos);
            // Re-render to cleanup
            this.setInputValue(input.value);
        }
        else {
            // If several commands
            // Get missing part of command name
            let commandsCommonPrefix = utils_1.strsCommonPrefix(commands);
            missing = commandsCommonPrefix.substring(insert.length);
            // Set new value
            input.value = input.value.substring(0, beg) +
                missing +
                input.value.substring(end);
            // Set new caret position
            pos += missing.length;
            this.setInputCaretPosition(pos);
            // Re-render to cleanup
            this.setInputValue(input.value);
        }
        ;
    }
    ;
    setInputCaretPosition(position) {
        if (!this.input) {
            return;
        }
        ;
        let { input } = this;
        input.selectionStart = position;
        input.selectionEnd = position;
    }
    ;
    setInputValue(value) {
        this.setState({ value });
    }
    ;
    getFilteredCommandsList() {
        let seek = this.getCurrentCommandPiece();
        let list = [];
        if (seek) {
            // Get filtered list if possible
            list = Input.commandsList.filter((item) => {
                return item.indexOf(seek) === 0;
            });
            // Check for values inside
            if (list.length) {
                return list;
            }
        }
        return [];
    }
    render() {
        const rect = this.input ? this.input.getBoundingClientRect() : null;
        const dropdown = rect && rect.top < 50 ? true : false;
        const suggest = !this.state.hideSuggest ? this.createSuggest(dropdown) : null;
        const textarea = this.createTextarea();
        return (React.createElement("div", { className: "Input" },
            suggest,
            textarea));
    }
    createSuggest(dropdown) {
        const { value } = this.state;
        const { theme } = this.props;
        const commands = this.getFilteredCommandsList();
        return (React.createElement(SuggestRoot, { value: value, commands: commands, dropdown: dropdown, theme: theme }));
    }
    createTextarea() {
        const { autoFocus } = this.props;
        const { rows, value } = this.state;
        return (React.createElement("textarea", { className: "cli", rows: rows, autoFocus: autoFocus, ref: e => {
                this.input = e;
                this.props.inputRef(e);
            }, value: value, onChange: this.onChange, onKeyDown: this.onKeyPress }));
    }
}
Input.commandsVocabulary = WavesConsoleAPI_1.WavesConsoleAPIHelp.texts;
Input.commandsList = Object.keys(WavesConsoleAPI_1.WavesConsoleAPIHelp.texts);
Input.commasAndQuotes = {
    '(': ')',
    '{': '}',
    '[': ']',
    '"': '"',
    "'": "'"
};
exports.Input = Input;
function SuggestRoot(props) {
    const { value, theme, commands } = props;
    // No need to go further
    if (!commands || !commands.length || !value) {
        return null;
    }
    return (React.createElement("div", { className: 'Suggest' + (theme ? ' Suggest_theme_' + theme : '') },
        React.createElement(SuggestList, Object.assign({}, props)),
        "\u00A0"));
}
function SuggestList(props) {
    // No need to go further
    if (!props.commands || !props.commands.length) {
        return null;
    }
    const commands = props.commands.map((item, index) => {
        return (React.createElement(SuggestItem, { key: 'commands-suggest-item-' + index, title: item }));
    });
    return (React.createElement("ul", { className: 'Suggest__list' + (props.dropdown ? ' Suggest__list_drop_down' : '') }, commands));
}
function SuggestItem(props) {
    return (React.createElement("li", { className: 'Suggest__item' + (props.selected ? ' Suggest__item_is_selected' : '') }, props.title));
}
