"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const run_1 = require("./run");
const WavesConsoleAPI_1 = require("../../WavesConsoleAPI");
const waves_transactions_1 = require("@waves/waves-transactions/");
const axios_1 = require("axios");
exports.updateIFrameEnv = (env) => {
    try {
        WavesConsoleAPI_1.WavesConsoleAPI.setEnv(env);
        const iframeWindow = run_1.getContainer().contentWindow;
        iframeWindow['env'] = env;
    }
    catch (e) {
        console.error(e);
    }
};
exports.bindAPItoIFrame = (consoleApi, console) => {
    const apiMethodWrappers = getApiMethodWrappers(consoleApi, console);
    try {
        const iframeWindow = run_1.getContainer().contentWindow;
        Object.keys(consoleApi)
            .forEach(key => {
            key in apiMethodWrappers
                ? iframeWindow[key] = apiMethodWrappers[key]
                : iframeWindow[key] = consoleApi[key];
        });
    }
    catch (e) {
        console.error(e);
    }
};
const getNetworkByte = (apiBase) => {
    return axios_1.default.get(`${apiBase}/addresses`)
        .then(res => {
        const address = res.data[0];
        const byte = waves_transactions_1.libs.marshall.serializePrimitives.BASE58_STRING(address)[1];
        return String.fromCharCode(byte);
    });
};
const getApiMethodWrappers = (consoleApi, console) => {
    return {
        broadcast: (tx, apiBaseParam) => __awaiter(this, void 0, void 0, function* () {
            const apiBase = new URL(apiBaseParam || WavesConsoleAPI_1.WavesConsoleAPI.env.API_BASE).href;
            const nodes = ['https://nodes.wavesplatform.com/', 'https://testnodes.wavesnodes.com/'];
            const pushExplorerLinkToConsole = (href) => {
                const htmlString = `<a href="${href}" target="_blank">Link to transaction in wavesexplorer</a>`;
                console.push({
                    html: true,
                    value: htmlString,
                    type: 'log',
                });
            };
            const generateExplorerLinkToTx = (networkByte, txId) => {
                return (networkByte === 'W')
                    ? `https://wavesexplorer.com/tx/${txId}`
                    : `https://wavesexplorer.com/testnet/tx/${txId}`;
            };
            const res = yield consoleApi.broadcast(tx, apiBase);
            if (nodes.includes(apiBase)) {
                const networkByte = apiBase === 'https://nodes.wavesplatform.com/'
                    ? 'W'
                    : 'T';
                const href = generateExplorerLinkToTx(networkByte, res.id);
                pushExplorerLinkToConsole(href);
            }
            else {
                try {
                    let networkByte = yield getNetworkByte(apiBase);
                    const isWavesNetwork = networkByte === 'W' || networkByte === 'T';
                    if (isWavesNetwork) {
                        const href = generateExplorerLinkToTx(networkByte, res.id);
                        pushExplorerLinkToConsole(href);
                    }
                }
                catch (e) {
                    console.log('Error occured during network byte check');
                }
            }
            return res;
        })
    };
};
